#!/bin/sh

TEMP_DIR="/tmp/"  # change to ~/tmp/ if your system won't let you build in /tmp/
SPUTNIK_DIR=$1    # "SPUTNIK_DIR" really is "the Kepler directory"

################################################################################
############# STEP 1: BUILD AND INSTALL KEPLER #################################
################################################################################

### cd to $TEMP_DIR directory, to avoid polluting the local directory
### with temporary files

OLD_DIR=$PWD
cd $TEMP_DIR

### download and untar Kepler

#ARCHIVE_NAME="kepler-1.1-snapshot-20071010-2144.tar.gz"
ARCHIVE_NAME="kepler-1.1-snapshot-20071031-1853.tar.gz"
#ARCHIVE_NAME=kepler-1.1-snapshot-20071001-1737.tar.gz

if [ -f $ARCHIVE_NAME ]
then
  echo "Skipping download - kepler archive already here"
else
  wget http://www.keplerproject.org/$ARCHIVE_NAME
fi
tar xvzf $ARCHIVE_NAME
cd kepler-1.1

### configure, build and install kepler into $SPUTNIK_DIR
./configure --prefix=$SPUTNIK_DIR --launcher=cgi --without-readline --enable-lua
make
make install

### return to the sputnik-wiki directory
cd $OLD_DIR

################################################################################
############# STEP 2: RE-CONFIGURE KEPLER, INSTALL SPUTNIK #####################
################################################################################

### rename cgi to sputnik.cgi
cp $SPUTNIK_DIR/bin/cgi sputnik.cgi

### copy the lua files to where they need to be
cp *.lua $SPUTNIK_DIR/share/lua/5.1/
cp examples/*.lua $SPUTNIK_DIR/share/lua/5.1/
cp dependencies/*.lua $SPUTNIK_DIR/share/lua/5.1
cp luasvn/*.so $SPUTNIK_DIR/lib/lua/5.1

### create and populate the initial data directory
$SPUTNIK_DIR/bin/lua5.1 make_data_dir.lua $SPUTNIK_DIR/etc/kepler/1.1/kepler_init.lua
$SPUTNIK_DIR/bin/lua5.1 make_config_file.lua $SPUTNIK_DIR/etc/kepler/1.1/kepler_init.lua $SPUTNIK_DIR
chmod -R a+rw $SPUTNIK_DIR
cp new_sputnik_config.lua $SPUTNIK_DIR/share/lua/5.1/sputnik_config.lua
cp cgilua.config.lua $SPUTNIK_DIR/etc/kepler/1.1/cgilua/config.lua
cp sputnik.lua $SPUTNIK_DIR/

