--DEBUG = true

debug = function() 
   -- do nothing
end
if DEBUG then
   debug = print
end

-------------------------------------------------------------------------------
-- Authentication -------------------------------------------------------------
-------------------------------------------------------------------------------

function make_authenticator (sputnik)

   local password_page = sputnik:get_page(sputnik.config.PASS_PAGE_NAME)
   password_page = sputnik:decorate_page(password_page)
   local passwords = sputnik:load_config(password_page.content)
   passwords = passwords.PASSWORDS or {}

   local function check_pw(username, password) 
      return md5.sumhexa(password) == passwords[username]
   end

   local function get_token(username) 
      return md5.sumhexa(username .. sputnik_config.SECRET_CODE) 
   end

   local function check_token(username, token)
      return get_token(username) == token
   end

   local function user_exists(username) 
      return passwords[username]
   end

   local function add_user(username, password) 
      passwords[username] = md5.sumhexa(password)
   end

   local function save(user)
      local new_values = {
         content = cosmo.fill(
            [=[    PASSWORDS = {}
                   $do_passwords[[
                   PASSWORDS["$user"]="$password"
            ]]]=],
            { do_passwords = function()
                 for user, password in pairs(passwords) do
                    cosmo.yield{user=user, password=password}
                 end
              end
            }
         ),
         author  = user,
         summary = "Added new user: "..user,
         minor   = "yes",
      }
      password_page = sputnik:update_page_with_params(password_page,
                                                      new_values)
      password_page = sputnik:decorate_page(password_page)
      sputnik:save_page(password_page)
   end

   local function authenticate_by_password(user, password) 
      if check_pw(user, password) then
	 return user, get_token(user)
      elseif user_exists(user) then 
	 return nil
      else
	 add_user(user, password)
	 save(user)
	 return user, get_token(user)
      end
   end

   local function authenticate_by_token(user, token)
      --[[if check_token(user, token) then
	 return user
      else
	 return nil
      end]]
      return user
   end

   return { 
      check_password = authenticate_by_password,
      check_token = authenticate_by_token 
   }
end





-------------------------------------------------------------------------------
-- A function to handle the CGI parameters (and do the authentication) --------
-------------------------------------------------------------------------------

function get_params (cgi, cookie, sputnik)

   local page_name, action
   if cgi.p then -- "nice" url
      page_name, action = sputnik:split(cgi.p, "%.")
   else
      page_name, action = cgi.page, cgi.action
   end

   if not page_name or page_name:len() == 0 then 
      page_name = sputnik.config.HOME_PAGE 
   end
   if not action then action = "show" end

   local params = {}
   for k, v in pairs(cgi) do
      params[k] = v
   end

   params.page = nil
   params.user = nil
   params.alleged_user = cgi.user
   params.page_name = page_name
   if action:len() > 0 then
      params.action = action
   else
      params.action = nil
   end

   params.user_from_cookie, params.auth_token = sputnik:split(cookie, "|")

   local auth = make_authenticator(sputnik)
   if cgi.user and cgi.user:len() > 0 then
      params.user, params.auth_token = auth.check_password(cgi.user, cgi.pw)
   elseif params.auth_token then
      params.user = auth.check_token(params.user_from_cookie, params.auth_token)
   else 
      params.user = nil
   end

   return params
end

--------------------------------------------------- "MAIN" ----------

require("lfs")              -- luaFileSystem
require("cgilua.cookies")
require("md5") 
require("cosmo")
require("wikistorage")
require("sputnik_class")
require("sputnik_actions")
require("sputnik_config")
require("sputnik_markup")

function run_sputnik(http_layer)

   -- http_layer is a cgilua adapter in our case, but we could theoretically plug in a different one

   local sputnik = sputnik_class.Sputnik:new(sputnik_config)

   local params = get_params(http_layer.cgi, 
                             http_layer.get_cookie(sputnik.config.COOKIE_NAME) or "", 
                             sputnik)
   local page = sputnik:get_page(params.page_name, params.version)
   page = sputnik:decorate_page(page, params)

   local action_function = page.actions[params.action or "show"]
                           or sputnik_actions.actions.action_not_found

   local content, content_type = action_function(page, params, sputnik)
   if params.user and params.auth_token then
      http_layer.set_cookie(sputnik.config.COOKIE_NAME, params.user.."|"..params.auth_token)
   end
   http_layer.set_content_type(content_type or "text/html")

   assert(content)       
   http_layer.write(content)
end

----- An adapter for CGILua -------------------------------------------------------------------

local http_layer = {}
http_layer.get_cookie = cgilua.cookies.get
http_layer.set_cookie = cgilua.cookies.set
http_layer.set_content_type = SAPI.Response.contenttype
http_layer.write = cgilua.put

if cgilua.requestmethod=="POST" then
   http_layer.cgi = cgilua.POST
else
   http_layer.cgi = cgilua.QUERY
end

----- End of the CGILua adapter ---------------------------------------------------------------


run_sputnik(http_layer)

      
	
		



